// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: NetArc.java
// Classes: NetArc
// Original Author: ics125b spring 1996
// $Id: NetArc.java,v 1.4 1996/09/17 00:42:01 jrobbins Exp $

package uci.graphedit;

import java.awt.*;
import java.io.*;

/** This class models an arc in our underlying connected graph model */

public abstract class NetArc extends NetPrimitive {

  /** The start and end ports of this arc. */
  private NetPort _sourcePort;
  private NetPort _destPort;

  /** Construct a new NetArc */
  public NetArc() { }

  /** Get and set methods */
  public void sourcePort(NetPort s) { _sourcePort = s; }
  public NetPort sourcePort() { return _sourcePort; }
  public void destPort(NetPort d) { _destPort = d; }
  public NetPort destPort() { return _destPort; }
  
  /** Connect the source and destination ports, iff they agree to
   * being connected (i.e., canConnectTo() returns true). Reply true
   * on success. This method is noramlly called after a new arc
   * instance is made. Maybe this behavior should be in a constructor,
   * but I want to use Class#newInstancel so constructors do not get
   * any arguments. */
  public boolean connect(NetPort s, NetPort d) {
    if (s.canConnectTo(d) && d.canConnectTo(s)) {
      sourcePort(s);		destPort(d);
      s.addArc(this); 		d.addArc(this);
      s.postConnect(d); 	d.postConnect(s);
      return true;
    }
    return false;
  }
  
  /** Remove this NetArc from the underlying connected graph model. */
  public void dispose() {
    System.out.println("disposing: " + toString());
    if (sourcePort() != null && destPort() != null) {
      _sourcePort.removeArc(this);
      _destPort.removeArc(this);
      // needs-more-work: assumes no parallel arcs!
      _sourcePort.postDisconnect(destPort());
      _destPort.postDisconnect(sourcePort());
    }
  }
  
  /** reply true iff the given port is my source port */
  public boolean isSource(NetPort prt) { return prt == sourcePort(); }
  
} /* end class NetArc */







